process.env.NODE_ENV = 'development';

var opn = require('opn')
var path = require('path')
var express = require('express')
var webpack = require('webpack')
var proxyMiddleware = require('http-proxy-middleware')
var webpackConfig = require('./conf/webpack.dev.conf')
var appConf = require('./conf/app.conf')
var utils = require('./utils')

utils.checkLoaderEnable(webpackConfig, 'eslintEnable', 'eslint-loader')
utils.checkLoaderEnable(webpackConfig, 'babelEnable', 'babel-loader')

// default port where dev server listens for incoming traffic
var port = process.env.PORT || appConf.devPort
// automatically open browser, if not set will be false
var autoOpenBrowser = true
// Define HTTP proxies to your custom API backend
// https://github.com/chimurai/http-proxy-middleware
var proxyTable = appConf.proxyTable

var app = express()
var compiler = webpack(webpackConfig)

var devMiddleware = require('webpack-dev-middleware')(compiler, {
  publicPath: webpackConfig.output.publicPath,
  quiet: true
})

var hotMiddleware = require('webpack-hot-middleware')(compiler, {
  log: () => {
  }
})
// force page reload when html-webpack-plugin template changes
compiler.plugin('compilation', function(compilation) {
  compilation.plugin('html-webpack-plugin-after-emit', function(data, cb) {
    hotMiddleware.publish({action: 'reload'})
    cb()
  })
})

// proxy api requests
if (proxyTable) {
  Object.keys(proxyTable).forEach(function(context) {
    var options = proxyTable[context]
    if (typeof options === 'string') {
      options = {target: options}
    }
    app.use(proxyMiddleware(options.filter || context, options))
  })
}

// handle fallback for HTML5 history API
app.use(require('connect-history-api-fallback')())

// serve webpack bundle output
app.use(devMiddleware)

// enable hot-reload and state-preserving
// compilation error display
app.use(hotMiddleware)

// serve pure static assets
var staticPath = path.posix.join(appConf.assetsPublicPath, appConf.assetsSubDirectory)
app.use(staticPath, express.static('./assets'))

var uri = 'http://localhost:' + port

devMiddleware.waitUntilValid(function() {
  console.log('> Listening at ' + uri + '\n')
})

module.exports = app.listen(port, function(err) {
  if (err) {
    console.log(err)
    return
  }

  // when env is testing, don't need open it
  if (autoOpenBrowser) {
    opn(uri)
  }
})
