/**
 * Copyright 2016 vip.com.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * </p>
 **/

package com.vip.saturn.job.sharding.task;

import com.vip.saturn.job.sharding.entity.Executor;
import com.vip.saturn.job.sharding.entity.Shard;
import com.vip.saturn.job.sharding.node.SaturnExecutorsNode;
import com.vip.saturn.job.sharding.service.NamespaceShardingService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 作业重排，移除所有executor的该作业shard，重新获取该作业的shards，finally删除forceShard结点
 */
public class ExecuteJobForceShardShardingTask extends AbstractAsyncShardingTask {

	private static final Logger log = LoggerFactory.getLogger(ExecuteJobForceShardShardingTask.class);

	private String jobName;

	public ExecuteJobForceShardShardingTask(NamespaceShardingService namespaceShardingService, String jobName) {
		super(namespaceShardingService);
		this.jobName = jobName;
	}

	@Override
	protected void logStartInfo() {
		log.info("Execute the {} with {} forceShard", this.getClass().getSimpleName(), jobName);
	}

	@Override
	public void run() {
		try {
			super.run();
		} finally {
			deleteForceShardNode();
		}
	}

	private void deleteForceShardNode() {
		try {
			String jobConfigForceShardNodePath = SaturnExecutorsNode.getJobConfigForceShardNodePath(jobName);
			if (curatorFramework.checkExists().forPath(jobConfigForceShardNodePath) != null) {
				curatorFramework.delete().forPath(jobConfigForceShardNodePath);
			}
		} catch (Throwable t) {
			log.error("delete forceShard node error", t);
		}
	}

	@Override
	protected boolean pick(List<String> allJobs, List<String> allEnableJobs, List<Shard> shardList,
			List<Executor> lastOnlineExecutorList, List<Executor> lastOnlineTrafficExecutorList) throws Exception {
		// 移除已经在Executor运行的该作业的所有Shard
		removeJobShardsOnExecutors(lastOnlineTrafficExecutorList, jobName);
		// 修正所有executor对该作业的jobNameList
		fixJobNameList(lastOnlineExecutorList, jobName);
		// 如果该作业是启用状态，则创建该作业的Shard
		if (allEnableJobs.contains(jobName)) {
			shardList.addAll(createShards(jobName, lastOnlineTrafficExecutorList));
		}

		return true;
	}
}
